#############################################################################
### OpenGL render engine for BlockIn
### Copyright 2004 Joseph Jezak
#############################################################################

#############################################################################
### This file is part of BlockIn.
###
### BlockIn is free software; you can redistribute it and/or modify
### it under the terms of the GNU General Public License as published by
### the Free Software Foundation; either version 2 of the License, or
### (at your option) any later version.
###
### BlockIn is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.
###
### You should have received a copy of the GNU General Public License
### along with BlockIn; if not, write to the Free Software
### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################


### FIXME ADD COLORS
### Color definitions
my @colors = ( 
  [0.0,0.0,0.0],	### Black
  [1.0,1.0,1.0],	### White
  [1.0,0.0,0.0],	### Red
  [0.0,1.0,1.0],	### Cyan
  [0.0,1.0,0.0],	### Green
  [1.0,0.5,0.0],	### Orange
  [0.0,0.0,1.0],	### Blue
  [1.0,0.0,1.0],	### Purple
  [1.0,1.0,0.0],	### Yellow
  [1.0,0.0,0.0],	### Red
  [0.0,1.0,1.0],	### Cyan
  [0.0,1.0,0.0],	### Green
  [1.0,0.5,0.0],	### Orange
  [0.0,0.0,1.0],	### Blue
  [1.0,0.0,1.0],	### Purple
  [1.0,1.0,0.0],	### Yellow
);

### Font definition
my $font = load_shapes("font",1);

###################################
### Load the shape sets
###
### Triplets 0,0,0 seperated by :
###################################
sub load_shapes{
  my ($shapeset, $hash) = @_;
  my %shapes;
  my @shapes;
  ### Open the shape data file
  open SHAPEDATA, "data/".$shapeset.".set" or die "Can't open the shape data file!  Make sure data/$shapeset.set exists!";
  ### Read in the shape data
  while(<SHAPEDATA>){
    chomp;
    ### Split on ;'s
    my @sd = split /;/;
    ### Make a shape out of this data this data
    my $shape = new Shape(shift @sd);
    foreach(@sd){
      $shape->add_cube($_);
    }
    $shapes{$shape->{name}} = $shape;
    push @shapes, $shape;
  }
  ### Close the shape data file
  close SHAPEDATA;
  return \%shapes if($hash);
  return \@shapes;
}

################################
### Cube Objects
################################
sub draw_cube{
  my ($x,$y,$z,$wireframe,$color,$alpha) = @_;
 
  ### Move the cube to the correct place
  glPushMatrix();
  glTranslate($x,$y,$z);
  
  ### Set the color
  $color = $colors[$color];
  glColor(@$color, $alpha) if($color && $alpha);
  glColor(@$color) if($color && !$alpha);
  
  ### Set wireframe or solid
  if($wireframe){ 
    glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
  }
  else{ 
    ### Offset the cube
    glPolygonOffset(-0.0005,0.0);
    glPolygonMode(GL_FRONT_AND_BACK,GL_FILL); 
  }
  
  ### Draw the cube
  glBegin(GL_QUADS);
    ### Top
    glVertex(0.0, 0.0, 1.0);
    glVertex(1.0, 0.0, 1.0);
    glVertex(1.0, 1.0, 1.0);
    glVertex(0.0, 1.0, 1.0);
    ### Bottom
    glVertex(0.0, 0.0, 0.0);
    glVertex(1.0, 0.0, 0.0);
    glVertex(1.0, 1.0, 0.0);
    glVertex(0.0, 1.0, 0.0);
    ### Front
    glVertex(0.0, 0.0, 0.0);
    glVertex(1.0, 0.0, 0.0);
    glVertex(1.0, 0.0, 1.0);
    glVertex(0.0, 0.0, 1.0);
    ### Back
    glVertex(0.0, 1.0, 0.0);
    glVertex(1.0, 1.0, 0.0);
    glVertex(1.0, 1.0, 1.0);
    glVertex(0.0, 1.0, 1.0);
    ### Left
    glVertex(0.0, 0.0, 0.0);
    glVertex(0.0, 1.0, 0.0);
    glVertex(0.0, 1.0, 1.0);
    glVertex(0.0, 0.0, 1.0);
    ### Right
    glVertex(1.0, 0.0, 0.0);
    glVertex(1.0, 1.0, 0.0);
    glVertex(1.0, 1.0, 1.0);
    glVertex(1.0, 0.0, 1.0);
  glEnd();
  ### Reset the color
  glColor(1.0,1.0,1.0);
  ### Reset the offset
  glPolygonOffset(0,0);
  glPopMatrix();
}

####################################################################
### Draw the mesh background and
### Screen background
####################################################################
sub draw_background{
  my ($width,$length,$depth) = @_;
  ### Color Lines
  glColor(0.0,1.0,0.0);
  ### The Grids are just lines
  glBegin(GL_LINES); 
    ### Bottom Grid
    for(my $i=0;$i<=$width;$i++){
      glVertex($i,0.0,$depth);
      glVertex($i,$length,$depth);
    }
    for(my $i=0;$i<=$length;$i++){
      glVertex(0.0,$i,$depth);
      glVertex($width,$i,$depth);
    }
    
    ### Right Wall Grid
    for(my $i=0;$i<=$width;$i++){
      glVertex($i,0.0,0.0);
      glVertex($i,0.0,$depth);
    }
    for(my $i=0;$i<=$depth;$i++){
      glVertex(0.0,0.0,$i);
      glVertex($width,0.0,$i);
    }
    
    ### Left Wall Grid
    for(my $i=0;$i<=$width;$i++){
      glVertex($i,$length,0.0);
      glVertex($i,$length,$depth);
    }
    for(my $i=0;$i<=$depth;$i++){
      glVertex(0.0,$length,$i);
      glVertex($width,$length,$i);
    }
    
    ### Top Wall Grid
    for(my $i=0;$i<=$length;$i++){
      glVertex($width,$i,0.0);
      glVertex($width,$i,$depth);
    }
    for(my $i=0;$i<=$depth;$i++){
      glVertex($width,0.0,$i);
      glVertex($width,$length,$i);
    }

    ### Bottom Wall Grid
    for(my $i=0;$i<=$length;$i++){
      glVertex(0.0,$i,0.0);
      glVertex(0.0,$i,$depth);
    }
    for(my $i=0;$i<=$depth;$i++){
      glVertex(0.0,0.0,$i);
      glVertex(0.0,$length,$i);
    }
   
  glEnd();
  glColor(1.0,1.0,1.0);
}

################################
### Draw the blocks in the field
################################
sub draw_field{
  my ($field) = @_;
  my $levels = $field->{field}; 
  my ($width, $length);
  ### Cycle through each level
  for(my $i=$#$levels;$i>=0;$i--){
    ### Cycle through each column
    $length = $$levels[$i];
    for(my $j=0;$j<$field->{length};$j++){
      $width = $$length[$j];
      for(my $k=0;$k<$field->{width};$k++){
	if($width & (2**$k)){
	  ### Draw wireframe around cube
	  draw_cube($k,$j,$i,1,0);
	  ### Colors are offset by 2 to get rid of black and white
	  draw_cube($k,$j,$i,0,$i+2);
	}
      }
    }
  }
  glColor(1.0,1.0,1.0);
}

################################################
### Rotates the vector 90 degrees on the X axis
################################################
sub rotate_x{
  my ($vector, $sign) = @_;
  die "Sign must be -1 or 1\n" unless(abs($sign) == 1);
  
  my @nvec;
  $nvec[0] = -$sign * $vector->{x};
  $nvec[1] = -$sign * $vector->{z};
  $nvec[2] = $sign * $vector->{y};
  
  return new Vector(x=>$nvec[0],y=>$nvec[1],z=>$nvec[2]);
}

###############################################
### Rotates the vector 90 degrees on the Y axis
###############################################
sub rotate_y{
  my ($vector, $sign) = @_;
  die "Sign must be -1 or 1\n" unless(abs($sign) == 1);
  
  my @nvec;
  $nvec[0] = $sign * $vector->{z};
  $nvec[1] = $vector->{y};
  $nvec[2] = -$sign * $vector->{x};
  
  return new Vector(x=>$nvec[0],y=>$nvec[1],z=>$nvec[2]);
}

###############################################
### Rotates the vector 90 degrees on the Z axis
###############################################
sub rotate_z{
  my ($vector, $sign) = @_;
  die "Sign must be -1 or 1\n" unless(abs($sign) == 1);
 
  my @nvec;
  $nvec[0] = -$sign * $vector->{y};
  $nvec[1] = $sign * $vector->{x};
  $nvec[2] = $vector->{z};

  return new Vector(x=>$nvec[0],y=>$nvec[1],z=>$nvec[2]);
}

#####################################
### Prints "Block" text to the screen
#####################################
sub print_text{
  my ($text, $x, $y, $z, $color) = @_;
  ### Back up the matrix
  glPushMatrix();
  ### Position the text
  glTranslate($x, $y, $z);
  ### Colorize the text
  glColor(1.0,1.0,1.0);

  ### Get the individual characters
  for(my $i=0;$i<length($text);$i++){
    ### Draw black outline
    $font->{uc(substr($text,$i,1))}->draw(1,1);
    ### Fill cube
    $font->{uc(substr($text,$i,1))}->draw(0,$color);
    
    ### Move to the next character
    glTranslate(6.0,0.0,0.0);
  }
  ### Restore the matrix
  glPopMatrix();
  glColor(1.0,1.0,1.0);
}

1;
