#######################################################
### Menu System for BlockIn
### Copyright 2004 Joseph Jezak
#######################################################

#############################################################################
### This file is part of BlockIn.
###
### BlockIn is free software; you can redistribute it and/or modify
### it under the terms of the GNU General Public License as published by
### the Free Software Foundation; either version 2 of the License, or
### (at your option) any later version.
###
### BlockIn is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.
###
### You should have received a copy of the GNU General Public License
### along with BlockIn; if not, write to the Free Software
### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################

################################
### Menu Parent Class
################################
sub Menu::new{
  my $class = shift;
  my %op = @_;
  my $self = {
    title	=> $op{title},	### Title of menu
    app		=> $op{app},	### Application refrence 
    event	=> $op{event},	### Event Queue
    config	=> $op{config}, ### Configuration
    selected	=> 0,		### Selected Item
    items	=> 0,		### Items in menu
    titlecolor	=> 2,		### Color of the title
  };
  bless $self, $class;
  return $self;
}

################################
### Draw menu
################################
sub Menu::draw{
  my ($self) = @_;
  my $menuitems = $self->{items};
  ### Draw background
  draw_background(100,100,0);
  ### Draw title
  print_text($self->{title},0,55,-1,$self->{titlecolor}); 
  ### Draw items
  for(my $i=0;$i<$#$menuitems+1;$i++){
    if($self->{selected} == $i){
      $$menuitems[$i]->draw(1);
    }
    else{
      $$menuitems[$i]->draw(0);
    }
  }
}

###########################
### Add an item to the menu
###########################
sub Menu::add_item{
  my ($self, $item) = @_;
  my $menuitems = $self->{items};
  ### Count the number of items and set the location for the next item
  $item->{y} = 40 - ($#$menuitems) * 6; ### Need to include the title offset 6 are MAGIC NUMBERS! MAKE THEM GO AWAY!
  
  ### Add the item into the array
  push @$menuitems,$item;
}

############################
### Increment menu selection
############################
sub Menu::inc{
  my ($self) = @_;
  my $items = $self->{items};
  if($self->{selected} == $#$items){
    $self->{selected} = 0;
  }
  else{
    $self->{selected}++;
  }
  print $self->{selected},"\n";
}

############################
### Decrement menu selection
############################
sub Menu::dec{
  my ($self) = @_;
  my $items = $self->{items};
  if($self->{selected} == 0){
    $self->{selected} = $#$items;
  }
  else{
    $self->{selected}--;
  }
  print $self->{selected},"\n";
}

###################
### Run the Menu
###################
sub Menu::do_menu{
  my ($self) = @_;
  ### App, configuration and event queue refrences
  my $app = $self->{app};
  my $event = $self->{event};
  my $config = $self->{config};

  my $done = 0;
  my $keydown = 0;
  my $timer = $app->ticks + 1000;

  ### Menu Loop
  while(!$done){
    ### Clear the screen
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    ### Reset the view
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
  
    ### Set the perspective
    gluPerspective(25.0, $$config{width}/$$config{height}, -10.0, 10.0);
    gluLookAt(40,30, 150,
  	      40,30, -10,
  	      0.0,1.0,0.0);

    ### Draw the menu
    $self->draw();
    $app->sync();
    
    ### Look for input
    #$event->pump();
    #$event->poll();
    $event->wait();

    ### Check for window closes
    exit(0) if($event->type == SDL_QUIT);

    ### Check for key presses
    if($event->type == SDL_KEYDOWN && $keydown == 0){
      ### Check for directionals
      $self->inc() if($event->key_sym == SDLK_UP);
      $self->dec() if($event->key_sym == SDLK_UP);
      
      ### Check for exits
      $done = 1 if ($event->key_sym == SDLK_ESCAPE);

      ### Set key rate clock
      $keydown = $app->ticks() + $$config{keyrate};
    }
    ### Check for key release
    elsif($event->type == SDL_KEYUP || $keydown < $app->ticks()){
      $keydown = 0;
    }
  }
}

################################
### Menu Item
################################
sub MenuItem::new{
  my $class = shift;
  my %op = @_;
  my $self = {
    x		=> 0,			### X Location
    y		=> 0,			### Y Location
    z		=> -1,			### Z Location
    text	=> $op{text},		### Text to display
    color	=> $op{color} || 6,	### Color of inactive text
    scolor	=> $op{scolor} || 8,	### Color of selected text
    action	=> $op{action},		### Action to do when selected
  };
  bless $self, $class;
  return $self;
}

##################
### Draw menu item
##################
sub MenuItem::draw{
  my ($self, $selected) = shift;
  if($selected){
    print "Drawing: ",$self->{text}," selected.\n";
    print_text($self->{text},$self->{x},$self->{y},$self->{z}, $self->{scolor}); 
  }
  else{
    print "Drawing: ",$self->{text}," unselected.\n";
    print_text($self->{text},$self->{x},$self->{y},$self->{z}, $self->{color}); 
  }
}

1;
